/* Bandwidth test    */
/* alan williams     */
/* Started on RISCiX */
/* moved to RISC OS  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

long size  = 1*1024*1024;   /* size of the data file */
long block = 16*1024;       /* block size used for xfers */
long rate  = 0;             /* data rate in ks-1 */
int  mode  = 0;             /* Mode=1, read =2, write */
int  loop  = 0;             /* Loop=0 don't, 1= do.*/
int  dump  = 0;             /* dump portability concerns */
long total = 0;             /* bytes transfered so far (in K) */
clock_t tstart,last,now;    /* used to time stuff */
float tick,len;             /* how long is a tick on this os? */

void *buf;

/* Time stuff */
void start()
{
  total=0;
  tstart=clock();
   /* start a time job */
}

void addblock()
{
  int ks;
  clock_t target;
  /* a block has just been written to the file */
  /* calculate the data rate being achieved    */
  /* calculate a pause if we are rate limited  */
  total=total+(block/1024);
  /* Time we have been running in seconds */
  len=tick*(clock()-tstart);

  if(len==0.0) len=0.00001; /* try to avoid div zero */
  ks= (float)total/len;
  printf("%d K/s\n",ks);
  /* how long was the last transfer? */

  if(rate){
    /* We are rate limited */
    if(ks>rate){
      /* we are going faster than requiered */
      /* Wait untill time*rate reaches the same total we are */
      /* allread up to. */


      while( (rate*tick*(clock()-tstart))<(total)){
        }
    }
  }
}


/* Write a big file out */
void makebigfile()
{
  FILE *f;
  long n,m;
  f=fopen("bwtest","wb");
  /* Find how many blocks make up the entire file */
  /* A bit careless about the proportion of the last */
  /* block actually filled but who cares? */

  start();
  m=size / block;
  printf("%d blocks each of %d make up the file of size %d \n",m,block,size);
  for(n=0; n<m;n++){
    fwrite(buf,block,1,f);
    addblock();
  }
  fclose(f);
}

/* Read a big file in */
void readbigfile()
{
  FILE *f;
  long n,m;
  f=fopen("bwtest","rb");
  if(f==NULL){
    printf("You need to creat a data file first.\n");
    exit(1);
    }

  start();
  m=size / block;
  printf("%d blocks each of %d make up the file of size %d \n",m,block,size);
  for(n=0; n<m;n++){
    fread(buf,block,1,f);
    addblock();
  }
  fclose(f);
}



int main(int argc,char *argv[])
{
  int i;
  tick=1.0/CLOCKS_PER_SEC;

  if(argc==1){
    printf("bw [-flags ...]\n");
    printf("   Alan's portable bandwidth testing tool\n");
    printf("   -l loop the test \n");
    printf("   -r read test \n");
    printf("   -w write test \n");
    printf("   -s n MB size of test file\n");
    printf("   -b n K  size of each transfer block (Use 16 on DOS)\n");
    printf("   -k n K/s how many k bytes per second to xfer\n");
    printf("   -o Dump platform characteristics.\n");
    printf("   Spaces between the flag and the number please.\n");
    printf("   -r/-w/-l first please (not last at any rate)\n");
    printf("   Meaningful results may not be obtained for tests\n");
    printf("   less that a couple of seconds long.\n");
    printf("\n");
    printf("   RISC OS Version compiled with Acorn C V5  8K\n");
    printf("   RISC iX Version compiled on 1.21c\n");
    printf("   DOS version compiled with Watcom C 10.5  30K\n");
    printf("   Netware NLM compiled with Watcom C 10.5\n");
    printf("   Spot the OS with the shared C library...");
    printf("\n");

    exit(1);
  }
  for(i=1;i<argc-1;i++){
    /* process the arg flags */
    printf("Doing arg %d which is %s and %s \n",i,argv[i],argv[i+1]);
    if(strcmp("-s",argv[i])==0) size  = atoi(argv[i+1])*1024*1024;
    if(strcmp("-b",argv[i])==0) block = atoi(argv[i+1])*1024;
    if(strcmp("-k",argv[i])==0) rate  = atoi(argv[i+1]);
    if(strcmp("-r",argv[i])==0) mode  = 1;
    if(strcmp("-w",argv[i])==0) mode  = 2;
    if(strcmp("-o",argv[i])==0) dump  = 1;
    if(strcmp("-l",argv[i])==0) loop  = 1;

  }

  printf("Using the following settings\n");
  printf("Mode  %d\n",mode);
  printf("Loop  %d\n",loop);
  printf("Size  %d\n",size);
  printf("Block %d\n",block);
  printf("Rate  %d\n",rate);
  printf("Tick  %f\n",tick);

  if(dump){
    /* os diagnostics dump mode */
    printf("On this OS:\n");
    printf("  sizeof(int) is %d\n",sizeof(int));
    printf("  sizeof(long) is %d\n",sizeof(long));
    printf("  sizeof(clock_t) is %d\n",sizeof(clock_t));
    printf("  CLOCKS_PER_SEC is %d\n",CLOCKS_PER_SEC);
    printf("\n");
  }

  buf=malloc(block);

  if(buf==NULL){
    printf("Failed to allocate buffer\n");
    exit(1);
  }

  do
  {
    if(mode == 1) readbigfile();
    if(mode == 2) makebigfile();
    printf("\n");
  }
  while (loop);
  return 0;
}

