/*
 * $Header: /smsa/bsd:sys/conf/param.c:bsd.riscix1_2  1.18  $
 * $Source: /smsa/bsd:sys/conf/param.c: $
 *
 * Copyright (c) 1988 Acorn Computers Ltd., Cambridge, England
 *
 * $Desc$
 * $Log:	param.c,v $
 * Revision 1.18  90/10/30  11:54:53  mark
 * Removed all the complex but useless fifoinfo stuff; instead
 * there is just one configurable item, fifo_limit which determines
 * the maximum number of fifos which may be in use at once.  This
 * is now by default a function of memory size.
 * 
 * Revision 1.17  90/10/28  20:01:56  mark
 * Adjusted calculations for ndquot (to avoid using the now-removed
 * NMOUNT value) and ncallout (to suit possible changes in sleep()
 * implementation).
 * 
 * Revision 1.16  90/08/02  14:31:55  mark
 * As promised in log for previous revision, the static allocation
 * option has been removed.
 * 
 * Revision 1.15  90/08/02  14:25:37  mark
 * Boot-time dynamic allocation arrives!  This file has lots of
 * #ifndef's to permit individual tables to be statically allocated
 * but the facility is really a hangover from a debugging attempt
 * and will be removed: if you want a particular size for some table,
 * initialise the nxxxx value statically either here or by adb on
 * the kernel image.
 * 
 * Revision 1.13  90/01/04  16:54:30  mark
 * Reversed Keith's change in the last version, but all the
 * names stat_xxxx have become xxxx_stat, so ps output isn't
 * so useless.
 * 
 * Revision 1.12  89/12/04  21:57:18  keith
 * Make the stat_xxxx structures static so that ps does not find them
 * when looking for wait channels. Nothing in the kernel should use them
 * directly anyway!
 * 
 * Revision 1.11  89/12/01  12:22:40  keith
 * Add "swap_interleave" to specify behaviour of primary swap area.
 * 
 * Revision 1.10  89/11/07  16:58:35  mark
 * Anything based on NQUOTA (a historical hangover) has been
 * removed.
 * 
 * Revision 1.9  89/10/19  16:57:26  keith
 * The sysconf.h is now in the build directory, not in sys/sysconf.h
 * 
 * Revision 1.8  89/10/18  11:04:56  keith
 * Major shuffling of the header files, which now occur in ../include/... and no .h
 * files remain in source directories. Header files must now be included by
 * '#include <XYZ/name.h>' where XYZ is a sub-directory. For parametric header
 * files from the build directory use '#include "name.h"'
 * 
 * Revision 1.7  89/09/26  18:52:42  keith
 * RISC iX + NFS 4.0
 * 
 * Revision 1.5  89/06/09  17:10:12  mark
 * Restructured for much better system tunability.
 * 
 * Revision 1.4  88/10/16  10:07:24  keith
 * Added nmbufs and removed "buffers", these are now allocated from within the u-area
 * pages. (See vm_pt.c)
 * 
 * Revision 1.3  88/06/18  19:09:24  beta
 * Acorn Unix initial beta version
 * 
 */

#if defined(KERN_RCS) && !defined(lint)
static char RcsId[] = "$Header: /smsa/bsd:sys/conf/param.c:bsd.riscix1_2  1.18  $";
#endif
/*
 * Copyright (c) 1980, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)param.c	7.1 (Berkeley) 6/5/86
 */

#ifndef lint
char copyright[] =
"@(#) Copyright (c) 1980, 1986 Regents of the University of California.\n\
 @(#) Copyright (c) 1988, 1989 Acorn Computers Limited.\n\
 @(#) All rights reserved.\n";
#endif

/*
 * Run-time !alterable! system parameters.
 * Compiled with -DHZ=xx -DTIMEZONE=x -DDST=x -DMAXUSERS=xx
 */

#include "sysconf.h"

#include <machine/pte.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/buf.h>
#include <sys/socket.h>
#include <sys/user.h>
#include <sys/map.h>
#include <sys/proc.h>
#include <sys/text.h>
#include <sys/vnode.h>
#include <sys/file.h>
#include <sys/callout.h>
#include <sys/clist.h>
#include <sys/cmap.h>
#include <sys/mbuf.h>
#include <sys/dnlc.h>
#include <sys/kernel.h>
#include <ufs/inode.h>
#include <ufs/quota.h>


int	hz		= HZ;
int	tick		= 1000000 / HZ;
int	tickadj		= 240000 / (60 * HZ);
struct	timezone tz	= {TIMEZONE, DST};
int	nmbclusters	= NMBCLUSTERS;


/*
 * Pointers to system data structures.
 * If defining data statically, data also.
 */
int		nproc 		= 0;
struct proc *	proc		= 0;
struct proc *	procNPROC	= 0;

int		ntext 		= 0;
struct text *	text		= 0;
struct text *	textNTEXT	= 0;

int		ninode 		= 0;
struct inode *	inode		= 0;
struct inode *	inodeNINODE	= 0;

int		nfile 		= 0;
struct file *	file		= 0;
struct file *	fileNFILE	= 0;

#ifdef QUOTA
int		ndquot 		= 0;
struct dquot *	dquot		= 0;
struct dquot *	dquotNDQUOT	= 0;
#endif

int		ncallout	= 0;
struct callout *callout		= 0;

int		nbuf 		= 0;
struct buf *	buf		= 0;

int		nswbuf 		= 0;
struct buf *	swbuf		= 0;

int		nswapmap	= 0;
struct map *	swapmap		= 0;

struct ncache *	ncache		= 0;
int		ncsize		= 0;

struct cblock *	cfree		= 0;
int		nclist		= 0;

struct cmap *	cmap		= 0;
struct cmap *	ecmap		= 0;

int nmbuf 			= 0;

int fifo_limit			= 0;

int swap_interleave = 0;	/* Set 1 => interleave primary swap area */

extern caddr_t permalloc (int bytes);
extern char *kern_end;

init_params (int physmem_meg)
{
    int mscale = (physmem_meg - 4) / 4;	/* memory grows in 4Mb units */
    int mx = mscale > 3 ? 3 : mscale;
    int kern_size, ncmap;
    /*
     * Size, allocate and initialise pointers to system data structures.
     */
#define ALLOC(p,n,type)	p = (type *)permalloc ((n) * sizeof (type))
#define ALLOC2(p,e,n,type) p = (type *)permalloc ((n) * sizeof (type)); e = &p[n]

    if (nproc == 0)
	nproc = 50 + mscale * 20;
    ALLOC2 (proc, procNPROC, nproc, struct proc);

    if (ntext == 0)
	ntext = 40 + mscale * 16;
    ALLOC2 (text, textNTEXT, ntext, struct text);

    if (ninode == 0)
	ninode = 120 + mscale * 50;
    ALLOC2 (inode, inodeNINODE, ninode, struct inode);

    if (nfile == 0)
	nfile = 160 + mscale * 100;
    ALLOC2 (file, fileNFILE, nfile, struct file);

#ifdef	QUOTA
    if (ndquot == 0)
	ndquot = ninode + 20 + 10 * mscale;
    ALLOC2 (dquot, dquotNDQUOT, ndquot, struct dquot);
#endif/*QUOTA*/

    if (ncallout == 0)
	/*
	 * ncallout calculation allows for many processes using isleep() or
	 * other functionality which involves sleeping waiting for a timeout
	 * to happen.
	 */
	ncallout = nproc * 4 / 5 + mscale * 20;
    ALLOC (callout, ncallout, struct callout);

    if (nbuf == 0)
    {
	static unsigned char n_buf[4] = { 30, 48, 60, 72 };
	nbuf = n_buf[mx];
    }
    ALLOC (buf, nbuf, struct buf);

    if (nswbuf == 0)
	nswbuf = 12 + 4 * mx;
    ALLOC (swbuf, nswbuf, struct buf);

    if (nswapmap == 0)
	nswapmap = 200 + mscale * 100;
    ALLOC (swapmap, nswapmap, struct map);

    if (ncsize == 0)
	ncsize = 100 + 40*mscale;
    ALLOC (ncache, ncsize, struct ncache);

    if (nclist == 0)
	nclist = 120 + 50*mscale;
    ALLOC (cfree, nclist, struct cblock);

    if (nmbuf == 0)
    {
	static int n_mbuf[4] = { 512, 1024, 1280, 1536 };
	nmbuf = n_mbuf[mx];
    }

    if (fifo_limit == 0)
	fifo_limit = 10 + 5 * mscale;

    /*
     * Compute number of core map entries as total physical mem -
     * current kernel size.  This is cautious (there are a couple of
     * reserved pages not included, and the core map itself is not
     * included), but we don't care about a little waste.
     */
    kern_size = (unsigned int )kern_end - ctob(KTLBASE);
    ncmap = (physmem_meg * (1024*1024/NBPG)) - (kern_size/NBPG) + 1;
    ALLOC2 (cmap, ecmap, ncmap, struct cmap);
}

/* EOF param.c */
