#
# $Header: /smsa/bsd:sys/conf/sysent.awk:bsd.riscix1_2  1.5  $
# $Source: /smsa/bsd:sys/conf/sysent.awk: $
#
# $Desc:$
# This awk script is used to process the syscall interface definition file "../conf/sysent.def"
# to produce files for the kernel, the library, and <sys/syscall.h> for the user.
# Using one definition file means that (hopefully) the three sets of files can be kept in
# step. The kernel Makefile automatically deletes "../include/sys/syscall.h" and replaces it
# with the newly generated one. If the makefile reminds you that the newly generated syscall.h
# is different from the one checked in DONT forget that the users (and library builder)
# cannot see the new copy until you have checked it in.
#
# $Log:	sysent.awk,v $
# Revision 1.5  90/04/05  08:10:37  mark
# Added handling for FORK and PIPE special calls, and added else
# clause to handle unknown types to avoid major inconsistencies which
# the normal $1 != number check would allow through (since an entry
# would have been missed in sysent.c, shifting the rest of the table
# out by 1!).
# 
# Revision 1.4  89/12/17  22:11:32  keith
# Add definition of OLD kernel procedures (if other than nullsys)
# 
# Revision 1.3  89/12/17  02:44:36  keith
# Fix wait/wait3 fiasco.
# 
# Revision 1.2  89/12/16  23:54:39  keith
# Remove incorrect (and literally) dangling else.
# 
# Revision 1.1  89/12/07  20:32:31  keith
# Initial revision
# 
BEGIN {
	number = 1
	print "#if !defined(SIMPLE)"						> "sysent.c"
	print "#  if defined(SYSCALLTRACE)"					> "sysent.c"
	print "#    define SIMPLE(name, argc, text) \t{ name, argc, text },"	> "sysent.c"
	print "#    if defined(NULL)"						> "sysent.c"
	print "#      undef NULL"						> "sysent.c"
	print "#    endif"							> "sysent.c"
	print "#    define NULL ((char *)0)\n"					> "sysent.c"
	print "#  else"								> "sysent.c"
	print "#    define SIMPLE(name, argc, text) \t{ name, argc       },"	> "sysent.c"
	print "#  endif\n"							> "sysent.c"
	print "#endif\n"							> "sysent.c"
	print "#if !defined(__sysent_h_)"					> "sysent.c"
	print "#  define __sysent_h_"						> "sysent.c"
	print "#  include \"sysent.h\""						> "sysent.c"
	print "#endif\n"							> "sysent.c"
	print "struct sysent sysent[] = {"					> "sysent.c"
	print "SIMPLE(nosys, 0, NULL)"						> "sysent.c"

	print "#if defined(SYSCALLTRACE)"					> "sysent.h"
	print "struct sysent { int (*sy_call)(); int sy_narg; char *sy_name;};" > "sysent.h"
	print "#else"								> "sysent.h"
	print "struct sysent { int (*sy_call)(); int sy_narg;               };" > "sysent.h"
	print "#endif\n"							> "sysent.h"
	print "extern int nullsys();"						> "sysent.h"

	print "#ifndef	__sys_syscall_h"					> "syscall.h"
	print "#define	__sys_syscall_h\n"					> "syscall.h"
}

/\/\*/ , /\*\//	{
	print									> "syscall.h"
	next
}

/^[^#]/ && (length > 0) {
	if (number != $1)
	{	err = sprintf("#error **** sysent definition table error: SYS_%s = %d, found on line %d\n", $5, $1, number)
		print err	> "sysent.c"
		print err	> "libent.h"
		printf "%s: %d: definition table error: expected entry %d, found '%s'\n", FILENAME, NR, number, $0
		exit 1
	}
	else
	{
		if ($2 == "UNUSED")
		{	printf "SIMPLE(nosys, 0, NULL)\n"			> "sysent.c"
		} else
		if ($2 == "SIMPLE")
		{	if (NF == 6)
			{	printf "ALIAS(%s, %s)\n", $6, $5		> "libent.h"
				$6 = "/" $6
			}
			if ($4 > 4)
				printf "COMPLX(%s, %d)\n", $5, $4		> "libent.h"
			else
				printf "SIMPLE(%s)\n", $5			> "libent.h"

			printf "#define SYS_%s %d\n", $5, number		> "syscall.h"
			printf "SIMPLE(%s, %d, \"%s%s\")\n", $3, $4, $5, $6	> "sysent.c"
			printf "extern int %s();\n", $3				> "sysent.h"
		} else
		if ($2 == "EXTCALL")
		{	if (NF == 6)
			{	printf "EXTCALL(%s)\n", $6			> "libent.h"
				$6 = "/" $6
			}
			printf "EXTCALL(%s)\n", $5				> "libent.h"

			printf "#define SYS_%s %d\n", $5, number		> "syscall.h"
			printf "SIMPLE(%s, %d, \"%s%s\")\n", $3, $4, $5, $6	> "sysent.c"
			printf "extern int %s();\n", $3				> "sysent.h"
		} else
		if  ($2 == "INTERN")
		{	printf "INTERN(%s)\n", $5				> "libent.h"

			printf "#define SYS_%s %d\n", $5, number		> "syscall.h"
			printf "SIMPLE(%s, %d, \"%s%s\")\n", $3, $4, $5, $6	> "sysent.c"
			printf "extern int %s();\n", $3				> "sysent.h"
		} else
		if  ($2 == "FORK")
		{	printf "FORK(%s)\n", $5					> "libent.h"

			printf "#define SYS_%s %d\n", $5, number		> "syscall.h"
			printf "SIMPLE(%s, %d, \"%s%s\")\n", $3, $4, $5, $6	> "sysent.c"
			printf "extern int %s();\n", $3				> "sysent.h"
		} else
		if  ($2 == "PIPE")
		{	printf "PIPE(%s)\n", $5					> "libent.h"

			printf "#define SYS_%s %d\n", $5, number		> "syscall.h"
			printf "SIMPLE(%s, %d, \"%s%s\")\n", $3, $4, $5, $6	> "sysent.c"
			printf "extern int %s();\n", $3				> "sysent.h"
		} else
		if ($2 == "OLD")
		{	printf "SIMPLE(%s, %d, \"OLD %s (deprecated)\")\n", $3, $4, $5	> "sysent.c"
			if ($3 != "nullsys") printf "extern int %s();\n", $3		> "sysent.h"
			printf "/* SYS_%s was %d, now deprecated */\n", $5, number	> "syscall.h"
		} else
		{
			err = "#error * sysent definition table error: unknown call type *"
			print err	> "sysent.c"
			print err	> "libent.h"
			printf "*** unrecognised call type `%s', entry %d ***\n", $2, number
			exit 1
		}
	}
	number++
}
END {
	print "};"								> "sysent.c"
	print "int\tnsysent = sizeof (sysent) / sizeof (sysent[0]);"		> "sysent.c"

	print "#endif/*__sys_syscall_h*/"					> "syscall.h"
}

