/*
 * $Header: /smsa/bsd:sys/dev/md/md.c:bsd.riscix1_2  1.8  $
 * $Source: /smsa/bsd:sys/dev/md/md.c: $
 *
 * Copyright (c) 1988 Acorn Computers Ltd., Cambridge, England
 *
 * $Desc$
 * $Log:	md.c,v $
 * Revision 1.8  91/02/18  16:50:08  kwelton
 * Now has a separate configuration file, as all properly written drivers
 * should.
 * 
 * Revision 1.7  89/10/30  18:08:43  mark
 * Tidied in small ways, some because of the new "devconf.h".
 * 
 * Revision 1.6  89/10/18  11:16:07  keith
 * Major shuffling of the header files, which now occur in ../include/... and no .h
 * files remain in source directories. Header files must now be included by
 * '#include <XYZ/name.h>' where XYZ is a sub-directory. For parametric header
 * files from the build directory use '#include "name.h"'
 * 
 * Revision 1.5  89/10/05  11:42:56  mark
 * Added "md_notconf" bodge to keep ld/ar/ranlib happy.
 * 
 * Revision 1.4  89/06/20  17:16:55  mark
 * Substantially redone for correctness - this module still depends
 * on "md.h", since it's more trouble than it's worth to separate
 * the code and data (no-one ever configures it in anyway).  The
 * source should be supplied to users - someone might like to play
 * with it.
 * 
 * Revision 1.3  88/06/17  20:11:49  beta
 * Acorn Unix initial beta version
 * 
 */

#if defined(KERN_RCS) && !defined(lint)
static char RcsId[] = "$Header: /smsa/bsd:sys/dev/md/md.c:bsd.riscix1_2  1.8  $";
#endif

#include "devconf.h"

#if NMD > 0

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/buf.h>
#include <sys/errno.h>

/*
 * DO NOT change this typedef without reflecting the
 * change in its twin in ../../conf/md_conf.c.
 */
typedef char block[DEV_BSIZE];

extern block m_disk[];
extern int md_nblocks;

int mdopen(dev, flag)
int dev, flag;
{
	if (minor(dev))
		return ENXIO;

	return 0;
}

int mdclose(dev, flag)
	int dev, flag;
{
	return 0;
}

mdstrategy(bp)
  struct buf *bp;
{
    int err = 0;
    if (minor(bp->b_dev))
	err = ENXIO;
    else
    {
	int blk = bp->b_blkno;
	int blks = (bp->b_bcount + DEV_BSIZE - 1) / DEV_BSIZE;
	int is_read = bp->b_flags & B_READ;
	if (blk > md_nblocks || (blk == md_nblocks && !is_read))
	    err = ENXIO;
	else if (blk == md_nblocks)
	    bp->b_resid = bp->b_bcount;		/* read 0 bytes */
	else
	{
	    int bcount;
	    if (blk + blks <= md_nblocks) 
		bcount = bp->b_bcount;
	    else
		bcount = (md_nblocks - blk) * DEV_BSIZE;
	    if (is_read)
		bcopy (&m_disk[blk][0], bp->b_un.b_addr, bcount);
	    else
		bcopy(bp->b_un.b_addr, &m_disk[blk][0], bcount);
	    bp->b_resid = bp->b_bcount - bcount;
	}
    }
    if (err)
    {
	bp->b_flags |= B_ERROR;
	bp->b_error = err;
    }	
    iodone(bp);
}

#endif/*NMD > 0*/

/* EOF md.c */
