/* -*-c-*- */
/* if_ea.h */
/* Header file for Atomwide ethernet interface code */

/*
 * Developed from Atomwide c.if_ea (RISC OS)
 * Developed from Acorn RISC-OS if_en.c
 * Developed from Acorn RISC-iX if_en.c
 */

/*
 * $Author: kwelton $
 * $Date: 93/03/23 17:37:48 $
 * $Source: /smsa/bsd:sys/include/armif/if_ea.h: $
 *
 * $Log:	if_ea.h,v $
 * Revision 1.1  93/03/23  17:37:48  kwelton
 * Initial revision
 * 
 * Revision 1.7  93/02/12  12:59:03  nicko
 * Statistics gathering redone.  DO_STATS switched on.  The exact stats
 * that get collected have be rationalized.
 * 
 * Revision 1.6  92/09/06  18:52:37  nicko
 * Added entry to make collision interupts conditional.
 * Tidied up statistic gathering so that detailed stats
 * are conditional.
 * 
 * Revision 1.5  92/06/27  11:07:46  nicko
 * Support for kbk 1.21b put in.  No major changes.
 * 
 * Revision 1.4  92/05/28  00:48:55  nicko
 * RSC headers done correctly
 * File brought up to date after being trashed by a stray copy.
 * 
 * Revision 1.3  92/05/27  23:39:07  nicko
 * Tweeked to try ro get the RCS headers to work
 * 
 */

#ifndef __armif_if_ea_h
#define __armif_if_ea_h

/*****************************************************************************
 *
 * Assorted defines and typedefs
 */

/* #define DEFAULT_PODULE */

#define EA_DEBUG		0
#define EA_DEBUG_TXINT          0
#define EA_DEBUG_RXINT          0
#define SHORT_MBUF_INFO         1
#define EA_WATCHTIME            10              /* Time (in seconds) between watchdog calls */
#define TEST_REENTERS           0               /* Test to see if the interupt code gets reentered (and panic) */
#define DO_ASSERTS		0		/* asserts may access volatiles */
#define NOCARDPRESENT		-1		/* found in hascard[] array */
#define UNUSED(x)		((x) = (x))	/* Suppress unused variable warning */
#define MAXXCBSLOTS		4
#define INIT_FIRST_PASS 	0
#define INIT_SECOND_PASS	1
#define IRQS_IRQV		1		/* IRQs via the IrqV chain */
#define IRQS_DEVICE		0		/* IRQs via the device sources */
#define DO_STATS		1		/* Gather statistics */
#define BITS			8		/* 16 or 8 only */
#define CRCExtra		4		/* fudge */
#define ENABLE_16COLL_INT	0		/* A 1 will switch off TX after collisions */

#define USE_ASM_COPY		1

#define CSTATE_FAULTY		1
#define CSTATE_UP		2
#define CSTATE_DOWN	        4

/* Imported/exported official numbers and strings */

#define CARDSTR 			"ea"		/* Official ethernet card type string */
#define EA_TYPE 			0xa4		/* Official product type code */

typedef volatile unsigned char uv_char;
typedef volatile unsigned int uv_int;
typedef volatile unsigned int sema;

/*****************************************************************************
 *
 * Error defines for errors that we can generate.
 */

#define ENOERR		0x00	/* No error */
#define EIFDOWN 	0x01	/* Interface not active */
#define EIFFAULTY	0x02	/* Interface hardware faulty */
/* #define ENOBUFS	0x03 */	/* Not enough available buffers */
/* #define ENOMEM	0x04 */	/* Not enough available memory */
#define ENOTANIF	0x05	/* Interface does not exist */
#define ETOOBIG 	0x06	/* Data packet too big */
#define EBUFERR 	0x07	/* Interface buffer memory error (details) */
#define EINTERR 	0x08	/* Interface interrupt error (details) */
#define ESWERR		0x09	/* Interface internal software error (details) */
#define EBADCALL	0x0a	/* Bad call */
#define ETXBLOCKED	0x0b	/* Transmission blocked */
#define EIRQTHREAD	0x0c	/* Cannot perform request within an interrupt */
#define ECOLLIDE	0x0d	/* Packet not sent due to excess collisions */



/*****************************************************************************
 *
 * Defines relating to ethernet packets.
 *
 * Just to remind everyone, an ethernet packet with tcp/ip information looks
 * something like this:
 *
 * DESTADDR		6 bytes
 * SRCADDR		6 bytes
 * PACKTYPE		2 bytes
 * IPHDR		20 bytes
 * TCPHDR		20
 * DATA
 *
 */

#define ETHERMTU	1500



/*
 * packet field lengths
 */

#define ETH_ADDRSIZE		6
#define ETH_PACKTYPESIZE	2
#define ETH_DATAMAXSIZE 	1500
#define ETH_TRAILCRCSIZE	4

/* @@@@ these sizes need revising to account for the 4 bytes of crc fudge */

#define ETH_HDRSIZE		(ETH_ADDRSIZE * 2 + ETH_PACKTYPESIZE) /* 14 */
#define ETH_MAXSIZE		(ETH_HDRSIZE + ETH_DATAMAXSIZE + ETH_TRAILCRCSIZE)
#define ETH_MINPACKSIZE		64
#define ETH_MINSIZE		(64 - ETH_HDRSIZE - ETH_TRAILCRCSIZE) /* 46 */
#define ETH_DATAMINSIZE		ETH_MINSIZE

#define PREFER_TX		0	 /* transmit interrupt */
#define PREFER_RX		1	 /* receive interrupt */
#define PREFER_LD		2	 /* packet into buffer */

/*****************************************************************************
 *
 * Defines relating to bits within cs8005 registers
 */

/*
 * Bits common to command and status register. In the command register, these
 * bits request activation of the indicated facility; in the status register,
 * these bits reflect the activation status of the indicated facilty.
 */

#define DMAIntEnable		0x0001	/* Wr: Enable. Rd: Is enabled */
#define RxIntEnable		0x0002	/* Wr: Enable. Rd: Is enabled */
#define TxIntEnable		0x0004	/* Wr: Enable. Rd: Is enabled */
#define BufferWindowEnable	0x0008	/* Wr: Enable. Rd: Is enabled */
#define AllIntEnable            0x000f

#define DMAInt			0x0010	/* Wr: Ack int. Rd: Stat of int */
#define RxInt			0x0020	/* Wr: Ack int. Rd: Stat of int */
#define TxInt			0x0040	/* Wr: Ack int. Rd: Stat of int */
#define BufferWindowInt 	0x0080	/* Wr: Ack int. Rd: Stat of int */
#define AllInt                  0x00f0

/*
 * Bits specific to the command register.
 */

#define SetDMAOn		0x0100
#define SetRxOn 		0x0200
#define SetTxOn 		0x0400
#define SetDMAOff		0x0800
#define SetRxOff		0x1000
#define SetTxOff		0x2000
#define FIFORead		0x4000
#define FIFOWrite		0x8000

/*
 * Bits specific to the status register.
 */
 
#define DMAOn			SetDMAOn
#define RxOn			SetRxOn
#define TxOn			SetTxOn
#define FIFOFull		0x2000
#define FIFOEmpty		0x4000
#define FIFODir 		0x8000		/* 1 == read, 0 == write */

/*
 * Bits within configuration register one
 */
 
#define BufferCodeMask		0x000f
#define DMABurstIntervalMask	0x0030
#define DMABurstLengthMask	0x00c0
#define StationMask		0x3f00
#define AddrMatchModeMask	0xc000

/* Bits to OR in to specify the buffer to access */

#define ReceiveAddr0		0x0000
#define ReceiveAddr1		0x0001
#define ReceiveAddr2		0x0002
#define ReceiveAddr3		0x0003
#define ReceiveAddr4		0x0004
#define ReceiveAddr5		0x0005
#define AddressPROM		0x0006
#define TransmitBufferLimit	0x0007
#define LocalBuffer		0x0008
#define InterruptVector 	0x0009

/* Bits to OR in to specify the DMA Burst Interval, in nanoseconds */

#define DBI_Continuous		0x0000
#define DBI_800 		0x0010
#define DBI_1600		0x0020
#define DBI_3200		0x0030

/* Bits to OR in to specify the DMA Transfer Length */

#define DTL_1			0x0000
#define DTL_4			0x0040
#define DTL_8			0x0080
#define DTL_16			0x00c0

/* Bits to OR in to specify which addesses to enable */

#define ENADDR0 		0x0100
#define ENADDR1 		0x0200
#define ENADDR2 		0x0400
#define ENADDR3 		0x0800
#define ENADDR4 		0x1000
#define ENADDR5 		0x2000

/* Bits to OR in to specify the Address Matching Mode */

#define AMM_SpecificOnly	0x0000
#define AMM_SpecificBroadcast	0x4000
#define AMM_SpecificBroadcastMulticast	0x8000
#define AMM_Promiscuous 	0xc000		/* Receive all packets */

/*
 * Bits within configuration register two.
 */
 
#define ByteSwap		0x0001		/* 0 = ARM/Intel, 1 = Motorola */
#define AutoUpdWPP		0x0002		/* Auto Update Write Protect Ptr */
#define CRCErrorEnable		0x0008
#define DribbleError		0x0010
#define ShortFrameEnable	0x0020
#define SlotSelect		0x0040
#define PreamSelect		0x0080
#define AddrLength		0x0100
#define RecCRC			0x0200
#define XmitNoCRC		0x0400
#define Loopback		0x0800
#define ActivityLED		0x1000		/* 0 = lit, 1 = unlit */
#define ResetCS8005		0x8000		/* Takes <= 4 microseconds */

/*****************************************************************************
 *
 * Defines relating to the buffer memory.
 */
 
#define PAGESIZE	256			/* 256 pages of 256 bytes */
#define TXPAGES 	(24 * 4)		/* Give 24k to tx space */
#define RXPAGES 	(40 * 4)		/* Give rest avail to rx space */
#define TXSIZE		(TXPAGES * PAGESIZE)	/* In bytes */
#define RXSIZE		(RXPAGES * PAGESIZE)	/* In bytes */
#define TXSTART 	0			/* Start of memory onwards */
#define TXLIMIT 	TXSIZE			/* Not suitable to write directly to cs8005 */
#define RXSTART 	(64 * 1024 - RXSIZE)	/* Calc start address */
#define RXLIMIT 	(64 * 1024)		/* End of buffer memory */
#define RXMINAVAIL	(ETH_MAXSIZE * 2)	/* At least 2 packets worth */
#define TXMINAVAIL	8			/* Enough for finished packet + leeway */
#define TXWRAP(x)	((x) >= TXLIMIT ? (x) - TXLIMIT : (x))
#define RXWRAP(x)	((x) >= RXLIMIT ? (x) - RXSIZE : (x))

/*****************************************************************************
 *
 * Defines to calculate addresses of register in a cs8005
 */

#define MEMC_BASE		0x03000000
#define REG_STRIDE		0x40
#define REG_HALF_STRIDE         0x04
#define SLOT_STRIDE		0x4000

#define ADDRFOR16(slot, reg)	((cs8005reg *)(MEMC_BASE + slot * SLOT_STRIDE + reg * REG_STRIDE))
#define ADDRFOR8(slot, reg)	((cs8005breg *)(MEMC_BASE + slot * SLOT_STRIDE + reg * REG_STRIDE))

#define ADDRFOR_CMD8(slot)	ADDRFOR8(slot, 0)
#define ADDRFOR_STAT8(slot)	ADDRFOR8(slot, 0)
#define ADDRFOR_CONF18(slot)	ADDRFOR8(slot, 1)
#define ADDRFOR_CONF28(slot)	ADDRFOR8(slot, 2)
#define ADDRFOR_WPP8(slot)	ADDRFOR8(slot, 3)
#define ADDRFOR_WINPTR8(slot)	ADDRFOR8(slot, 4)
#define ADDRFOR_RXPTR8(slot)	ADDRFOR8(slot, 5)
#define ADDRFOR_TXPTR8(slot)	ADDRFOR8(slot, 6)
#define ADDRFOR_DMAPTR8(slot)	ADDRFOR8(slot, 7)

#define ADDRFOR_CMD16(slot)	ADDRFOR16(slot, 0)
#define ADDRFOR_STAT16(slot)	ADDRFOR16(slot, 0)
#define ADDRFOR_CONF116(slot)	ADDRFOR16(slot, 1)
#define ADDRFOR_CONF216(slot)	ADDRFOR16(slot, 2)
#define ADDRFOR_WPP16(slot)	ADDRFOR16(slot, 3)
#define ADDRFOR_WINPTR16(slot)	ADDRFOR16(slot, 4)
#define ADDRFOR_RXPTR16(slot)	ADDRFOR16(slot, 5)
#define ADDRFOR_TXPTR16(slot)	ADDRFOR16(slot, 6)
#define ADDRFOR_DMAPTR16(slot)	ADDRFOR16(slot, 7)


/*****************************************************************************
 *
 * Structures relating to packet transmission and receiption
 */

/* Format of packet transmit header in buffer memory */

typedef struct
{
    u_char mptr;		/* msb of next packet ptr */
    u_char lptr;		/* lsb of next packet ptr */
    u_char txcmd;		/* transmit command byte */
    u_char txstat;		/* transmit status byte */
} txhdr;

#define TX_BABBLE		0x01	/* txcmd flags */
#define TX_COLLINT		0x02
#define TX_16COLLINT		0x04
#define TX_SUCCINT		0x08
#define TX_DATAFOLLOWS		0x20
#define TX_CONTINUE		0x40
#define TX_MUSTBESET		0x80
#define TX_DONE 		0x80	/* in txstat, not txcmd */
#define TXHDRSIZE		sizeof(txhdr)

/* Format of packet receipt header in buffer memory */

typedef struct
{
    u_char mptr;		/* msb of next packet ptr */
    u_char lptr;		/* lsb of next packet ptr */
    u_char rxinfo;		/* receive information byte */
    u_char rxstat;		/* receive status byte */
} rxhdr;

#define RX_DATAFOLLOWS		0x40	/* rxinfo */
#define RX_MUSTBECLEAR		0x80
#define RX_BABBLE		0x01	/* rxstat */
#define RX_CRCERROR		0x02
#define RX_DRIBBLEERROR 	0x04
#define RX_SHORTPACKET		0x08
#define RX_DONE 		0x80
#define RXHDRSIZE		sizeof(rxhdr)

/*****************************************************************************
 *
 * Top level structures.
 */

/*
 * For 8 bit cards
 */

typedef volatile unsigned char cs8005breg;
#define LO(x)		((x) & 0xff)
#define HI(x)		(((x) >> 8) & 0xff)
#define lohalf		0
#define hihalf		REG_HALF_STRIDE

/*
v * For 16 bit cards
 */

typedef volatile unsigned int cs8005reg;		/* Write on top half, read on bottom half */
#define shup(x)		(((unsigned int)(x)) << 16)	/* Like this */
#define mask(x)		(*(x) & 0xffff)			/* The read masking */

/*
 * The order of these fields is known by ea_info. All fields u_ints.
 */

enum
{
     actints,			/* actual number of interrupts */

     txbabbles,			/* number babble (oversized) tx attempts */
     xscolls,			/* number tx 16/failed collisions */
     txbytes,			/* total data bytes transmitted (wraps!) */
     txmbufs,			/* tx frame droped for lack of mbuf */
     txmemory,			/* tx frames not sent due to lack of buffer memory */

     rxbytes,			/* total data bytes received (wraps!) */
     rxcrcerrors,		/* number crc errors on rx packets */
     rxbabbles,			/* number babble rx */
     rxshorts,			/* number short rx packets */
     rxdribbles,		/* number rx dribble error packets */
     rxmbufs,			/* rx frames abandonded due to lack of mbufs */
     rxframe,			/* rx frames whose frame type not known */
     rxqueue,			/* Protocol input queue full */

#if EA_DEBUG
     freceived,
     ftransmitted,
     floaded,
     rxups,
     txups,
     txpps,
     tx1,			/* tx early exit */
     tx2,			/* tx early exit */
     tx3,			/* tx early exit */
     tx4,
     tx5,
     collnotdown,
#endif

     total_stats
};

typedef struct cs8005;		/* Dunno why this is necessary */

typedef struct
{
    void (*fast_read)      (void *memoryptr, void *regptr, int bytecount);
    void (*fast_write)     (void *memoryptr, void *regptr, int bytecount);
    void (*slow_read)      (void *memoryptr, void *regptr, int bytecount);
    void (*slow_write)     (void *memoryptr, void *regptr, int bytecount);
    void (*setup_write_tx) (struct cs8005 *csp, u_int ptr);
    void (*setup_write_rx) (struct cs8005 *csp, u_int ptr);
    void (*setup_read_tx)  (struct cs8005 *csp, u_int ptr);
    void (*setup_read_rx)  (struct cs8005 *csp, u_int ptr);
} fnptr_t;

/*
 * Have one of these structures per card present.
 *
 * Don't forget that there are various restrictions on when some of these
 * values can be read and written;
 */

typedef struct cs8005
{
    /* Ethernet common part */
    struct arpcom ea_ac;
#define ea_if ea_ac.ac_if
#define etheraddr ea_ac.ac_enaddr

    /* general */
    
    u_char slot;		/* xcb slot this card is in */
    u_char unit;		/* unit number (0..) */
    u_char bits;		/* data bus size: 8 or 16 only */
    
    /* Interrupt related information -- volatile !! */
    
    uv_char cstate;		/* Card state */

    sema queue_lock;		/* Someone owns the head of the queue (and rights to the card) */
    sema txpending;		/* A tx needs dealing with */
    sema rxpending;		/* An rx needs dealing with */

    /* These function pointers represent most of the 8/16 bit differences */

    fnptr_t fns;

    /* direct pointers into the cs8005 chip itself -- volatile !! */

    union
    {
	struct
	{

	    /* Not quite correct with bufferr, as only use the low
	       address of this, so use same value in 8 and 16 bit
	       modes, and pass this on to appropriate assembler
	       routine anyway.
	     */

#define cmdr statr
	    cs8005breg *statr;		/* Ptr to command/status register */
	    cs8005breg *conf1r;		/* Ptr to configuration #1 register */
	    cs8005breg *conf2r;		/* Ptr to configuration #2 register */
	    cs8005breg *wppr;		/* Ptr to write protect pointer register */
	    cs8005breg *bufferr; 	/* Ptr to buffer window register */
	    cs8005breg *dmaptrr;	/* Ptr to dma ptr register */
	    cs8005breg *txptrr;		/* Ptr to transmit pointer register */
	    cs8005breg *rxptrr;		/* Ptr to receive pointer register */
	} bits8;

	struct
	{
	    cs8005reg *statr;
	    cs8005reg *conf1r;
	    cs8005reg *conf2r;
	    cs8005reg *wppr;
	    cs8005reg *bufferr;
	    cs8005reg *dmaptrr;
	    cs8005reg *txptrr;
	    cs8005reg *rxptrr;
	} bits16;
    } width;

    /* values associated with such registers -- volatile !! */
    
    /* always: int enable bits */
    /* never: int ack bits on/off bits fifo dir */
    
    uv_int cmd;			/* My last used value for command register */
    uv_int conf1;		/* My last used value for configuration #1 register */
    uv_int conf2;		/* My last used value for configuration #2 register */
    
    struct
    {
	sema free;		/* Inc by tx swi, dec by tx interrupt */
	u_int current;		/* Pointer to currently being processed txhdr */
	u_int end;		/* Pointer to null header; where to add at */
    } tx;
    
    struct
    {
	u_int current;		/* Pointer to next rxhdr to process */
    } rx;
    
#if DO_STATS
    u_int stats[total_stats];
#endif
} cs8005;



/*****************************************************************************/
/* Finally, some forward declarations.
 */
void fast_read8       (void *memoryptr, void *regptr, int bytecount);
void fast_write8      (void *memoryptr, void *regptr, int bytecount);
void slow_read8       (void *memoryptr, void *regptr, int bytecount);
void slow_write8      (void *memoryptr, void *regptr, int bytecount);
void setup_write_tx8  (struct cs8005 *csp, u_int ptr);
void setup_write_rx8  (struct cs8005 *csp, u_int ptr);
void setup_read_tx8   (struct cs8005 *csp, u_int ptr);
void setup_read_rx8   (struct cs8005 *csp, u_int ptr);

void fast_read16      (void *memoryptr, void *regptr, int bytecount);
void fast_write16     (void *memoryptr, void *regptr, int bytecount);
void slow_read16      (void *memoryptr, void *regptr, int bytecount);
void slow_write16     (void *memoryptr, void *regptr, int bytecount);
void setup_write_tx16 (struct cs8005 *csp, u_int ptr);
void setup_write_rx16 (struct cs8005 *csp, u_int ptr);
void setup_read_tx16  (struct cs8005 *csp, u_int ptr);
void setup_read_rx16  (struct cs8005 *csp, u_int ptr);

static void reflect_csp(cs8005 *csp);
static void write_null_tx_header(cs8005 *csp, u_int ptr);

int eainit(int);
int eareset(int);
int eaioctl();
int eaoutput();

static void transmitted(cs8005 *csp);
static void received(cs8005 *csp);
static void loading(cs8005 *csp, struct mbuf *m0);

static void handle_ints_and_queue(cs8005 *csp, int min_pl);

void ea_irq_handler(cs8005 *csp); 

#if EA_DEBUG
static void dump_mbuf(struct mbuf*mp);
eawatch(int);
#endif /* EA_DEBUG */

#endif	/*  __armif_if_ea_h */

/* EOF if_ea.h */

