/*
 * $Header: /smsa/bsd:sys/SYSTEM/ps_conf.c:bsd.riscix1_2  1.7  $
 * $Source: /smsa/bsd:sys/SYSTEM/ps_conf.c: $
 *
 * Copyright (c) 1989 Acorn Computers Ltd., Cambridge, England.
 *
 * $Log:	ps_conf.c,v $
 * Revision 1.7  90/09/07  10:59:13  mark
 * Renamed "pv" and "supported", to reduce potential conflicts.
 * 
 * Revision 1.6  90/02/09  11:53:47  kwelton
 * Put in support for Optical Memory devices - use same routines as Direct
 * Access.
 * 
 * Revision 1.5  89/10/31  14:38:33  mark
 * Configuration now controlled by "devconf.h" not "ps.h".
 * Also re-structured stuff so dependency generation is
 * independent of configuration.
 * 
 * Revision 1.4  89/10/18  11:19:21  keith
 * Major shuffling of the header files, which now occur in ../include/... and no .h
 * files remain in source directories. Header files must now be included by
 * '#include <XYZ/name.h>' where XYZ is a sub-directory. For parametric header
 * files from the build directory use '#include "name.h"'
 * 
 * Revision 1.3  89/07/04  08:44:02  mark
 * Added suitable type casts for values in null_pv.
 * 
 * Revision 1.2  89/07/04  07:52:24  mark
 * Added "#if PSC > 0" around the declarations.
 * 
 * Revision 1.1  89/05/26  15:21:15  kwelton
 * Initial revision
 * 
 *
 */
 
#if defined(KERN_RCS) && !defined(lint)
static char RcsId[] = "$Header: /smsa/bsd:sys/SYSTEM/ps_conf.c:bsd.riscix1_2  1.7  $";
#endif

#include "devconf.h"
#include "ps_conf.h"

#include <sys/errno.h>
#include <dev/ps_driver.h>
#include <dev/ps_direct.h>
#include <dev/ps_sequential.h>

#if PSC > 0

int ps_supported = 0
#ifdef DIRECT_ACCESS
                       | 1 << DC_DIRECT_ACCESS
#endif
#ifdef SEQUENTIAL_ACCESS
                       | 1 << DC_SEQUENTIAL_ACCESS
#endif
#ifdef PRINTER
                       | 1 << DC_PRINTER
#endif
#ifdef PROCESSOR
                       | 1 << DC_PROCESSOR
#endif
#ifdef WRITE_ONCE
                       | 1 << DC_WRITE_ONCE
#endif
#ifdef READ_ONLY
                       | 1 << DC_READ_ONLY
#endif
#ifdef OPTICAL_MEMORY
    		       | 1 << DC_OPTICAL_MEMORY
#endif
                       ;

static int ps_null()
{
    printf("scsi device class error\n");
    return ENODEV;
}

static Proc_Vec null_pv = 
{
    (dev_init)ps_null,
    (dev_open)ps_null,
    (dev_close)ps_null,
    (dev_strategy)ps_null,
    (dev_ioctl)ps_null,
    (dev_size)ps_null,
    (dev_cdb)ps_null
};

Proc_Vec *ps_pv[8] = {
#ifdef DIRECT_ACCESS
  &direct,
#else
  &null_pv,
#endif
#ifdef SEQUENTIAL_ACCESS
  &sequential,
#else
  &null_pv,
#endif
#ifdef PRINTER
  &printer,
#else
  &null_pv,
#endif
#ifdef PROCESSOR
  &processor,
#else
  &null_pv,
#endif
#ifdef WRITE_ONCE
  &write_once,
#else
  &null_pv,
#endif
#ifdef READ_ONLY
  &read_only,
#else
  &null_pv,
#endif
  &null_pv,			/* non-existent class 7 */
#ifdef OPTICAL_MEMORY
  &direct
#else
  &null_pv
#endif
};

#endif PSC > 0

/* EOF ps_conf.c */
