/*
 * $Header: /smsa/bsd:sys/SYSTEM/compileversion.c:bsd.riscix1_2  1.7  $
 * $Source: /smsa/bsd:sys/SYSTEM/compileversion.c: $
 *
 * Copyright (c) 1988 Acorn Computers Ltd., Cambridge, England
 *
 * $Desc$
 * $Log:	compileversion.c,v $
 * Revision 1.7  90/05/04  11:08:10  mark
 * Hacked in an optional (must be first) -n flag; this permits
 * the special #xxx number to be explicitly given, rather than
 * derived from ~/.vmunix_number.  Useful if the latter isn't
 * accessible for any reason.
 * 
 * Revision 1.6  89/10/23  19:41:57  keith
 * exec the actual compiler given, not just "cc"
 * 
 * Revision 1.5  89/10/18  11:20:00  keith
 * Major shuffling of the header files, which now occur in ../include/... and no .h
 * files remain in source directories. Header files must now be included by
 * '#include <XYZ/name.h>' where XYZ is a sub-directory. For parametric header
 * files from the build directory use '#include "name.h"'
 * 
 * Revision 1.4  89/10/05  11:54:27  mark
 * A few tweaks to keep the compiler happy; also added a hack
 * to allow the .vmunix_number file to be found in more than
 * one place if desired - this is compiled on #ifdef MARKS_HACK.
 * 
 * Revision 1.3  89/09/25  09:29:32  mark
 * The "compiler" parameter can now be more than just the name of
 * the program, e.g. it can contain leading arguments to the compiler.
 * 
 * Revision 1.2  88/10/25  12:30:48  keith
 * Fixed the -r option.
 * 
 * Revision 1.1  88/10/18  16:45:38  keith
 * Initial revision
 * 
 * Revision 1.1  88/09/21  03:06:27  keith
 * Initial version
 * 
 */

#if defined(TOOL_RCS) && !defined(lint)
static char RcsId[] = "$Header: /smsa/bsd:sys/SYSTEM/compileversion.c:bsd.riscix1_2  1.7  $";
#endif

#include <sys/types.h>
#include <sys/param.h>
#include <stdio.h>
#include <pwd.h>

extern char *getenv ();
extern char *getwd ();
extern char *ctime ();
extern char *index ();
extern char *rindex ();

static char *program;

main(argc, argv)
     int argc;
     char *argv[];
{
	char *compiler;
	time_t now = time(0);
	char *objectnumber_define;
	char objectversion_define[256];
	char name_define[256];
	char date_string[26];
	char version_text[256];
	char buffer[256*5];
	char *p, *q;
	char *argvec[100];
	char **ap = argvec;
	char *sep;
	char product_name[256];
	char release_name[256];
	char temp[MAXPATHLEN];
	char flavour[MAXPATHLEN];
	int special_number = 0;

	p = rindex(getwd(temp), '/');
	if (p) {
		if (strcmp("SYSTEM", p+1) == 0) {
			flavour[0] = 0;
		} else {
			sprintf(flavour, " (%s)", p+1);
		}
	}
	program = argv[0];
	if (strncmp (argv[1], "-n", 2) == 0)
	{
	    char *np;
	    ++argv; --argc;
	    if (argv[0][2] == '\0')
	    {
		++argv; --argc;
		np = &argv[0][0];
	    }
	    else
		np = &argv[0][2];
	    special_number = atoi (np);
	}
	compiler = argv[1];
	if (argc >= 3 && argv[2][0] == '-' && argv[2][1] == 'r') {
		char pb[256];
	  	FILE *pf = popen("getversion 1/", "r");
		if (pf == NULL) {
			perror("Unable to open pipe to \"getversion\"");
			exit(1);
		}
		if (fgets(pb, 256, pf) != NULL) {
			p = pb + strlen(pb);
			while (*p != '/' && p > pb)
				p--;
			if (*p != '/') {
				fprintf(stderr,
					"%s: Mangled version string '%s'\n",
					program, pb);
				exit(1);
			}
			sprintf(objectversion_define, "-DOBJECTVERSION=%d", atoi(++p) + 1);
		} else
			sprintf(objectversion_define, "-DOBJECTVERSION=%d", 1);
		objectnumber_define = "-DOBJECTNUMBER=1";
		pclose(pf);
		argc--;
		argv++;
	} else {
	  	objectnumber_define = "-DOBJECTNUMBER=0";
		objectversion_define[0] = '\0';
	}
	if (argc < 3) {
	  	fprintf(stderr, "Usage: %s compiler [-r] \"Product Name\" [ \"Release Name\" ]\n", program);
		exit(2);
	}
	*ap++ = compiler;
	/*
	 * We now allow the compiler parameter to contain arguments to
	 * be prefixed to the cc command: check for this case by
	 * looking for the separating space.
	 */
	if ((p = index (compiler, ' ')) != 0)
	{
	    char buffer[1023+1];		/* to preserve original parameter */
	    if (strlen (compiler) > 1023)
	    {
	  	fprintf(stderr, "%s: compiler definition too long\n", program);
		exit(2);
	    }		
	    strcpy (buffer, compiler);		/* for possible errors */
	    do 
	    {
		/* At this point, p is always pointing at a space. */
		*p++ = '\0';			/* terminate preceding string */
		while (*p == ' ')		/* skip any further spaces */
		    ++p;
		if (*p)				/* if not at end of parameter.. */
		{
		    int c = *p;
		    if (c == '"' || c == '\'')
		    {
			/* quoted argument */
			*ap++ = ++p;		/* actual arg starts after quote */
			while (*p && *p != c)	/* look for end-quote or end-parameter */
			    ++p;
			if (*p == '\0' ||
			    (p[1] != '\0' && p[1] != ' '))
			{
			    /* unmatched quote, or rubbish after end-quote */
			    fprintf (stderr,
				     "%s: error in compiler parameter \"%s\"\n",
				     program, buffer);
			    exit (2);
			}
			*p++ = '\0';
		    }
		    else
		    {
			/* unquoted arg */
			*ap++ = p;		/* arg starts now */
			/* look for next space or end of parameter */
			do
			    ++p;
			while (*p && *p != ' ');
		    }
		}
	    } while (*p);
	}
  /* If product_name has %s in it expand with (`basename pwd`) */
	sprintf(product_name, argv[2], flavour);
	if (argc > 3) {
		sprintf(release_name, "%s", argv[3]);
	} else {
		struct passwd *pw = getpwuid(getuid());
	  	sprintf(release_name, "%s #%d special:",
			pw->pw_name, 
			special_number > 0 ? 
			special_number : next_number(pw->pw_dir));
 	}
	sprintf(name_define, "-DRELEASENAME=\"%s\"", release_name);
	*ap++ = name_define;
	*ap++ = objectnumber_define;
	if (objectversion_define[0])
		*ap++ =	objectversion_define;
	p = ctime(&now);
	q = date_string;
	while (*p && *p != '\n')
		*q++ = *p++;
	*q = '\0';
	sprintf(version_text, "%s %s made %s",
		product_name, release_name, date_string);

	q = buffer;
	sprintf(buffer, "-DVERSIONSTRING=");
	q += strlen(buffer);
	p = version_text;
	sep = " ";
	do {
		sprintf(q, "%s0x%02x", sep, *p++);
		sep = ",";
		q += 5;
	} while (*(p-1));
	*ap++ = buffer;
	*ap++ = "-c";
	*ap++ = "version.c";
	*ap++ = 0;
	execvp(compiler, argvec);
  	fprintf(stderr, "%s: Unable to exec compiler '%s'\n", program, compiler);
	exit(3);
}

next_number(name)
     char *name;
{
	char path[1024];
	int number;
	FILE *f;

	sprintf(path, "%s/.vmunix_number", name);
#ifdef	MARKS_HACK
retry:
#endif/*MARKS_HACK*/
	number = 0;
	if ((f = fopen(path, "r+")) == NULL) {
		f = fopen(path, "w");
		number = 1;
	}
	if (f == NULL) {
#ifdef	MARKS_HACK
	    	char *home;
#endif/*MARKS_HACK*/
		fprintf(stderr, "%s: Unable to read/create ", program);
		perror(path);
#ifdef	MARKS_HACK
		home = getenv ("HOME");
		if (home != NULL && strcmp (home, name) != 0)
		{
		    sprintf(path, "%s/.vmunix_number", home);
		    fprintf(stderr, "-- Retrying with %s\n", path);
		    goto retry;
		}
#endif/*MARKS_HACK*/
		exit(997);
	}
	if (number == 0) {
		char old_number[10];
		fgets(old_number, 10, f);
		number = atoi(old_number);
		if (number == 0) {
			fprintf(stderr, "%s: Unable to read number in %s", program, path);
			exit(996);
		}
		number++;
	}
	rewind(f);
	fprintf(f, "%4d\n", number);
	fclose(f);
	return (number);
}

