/*
 * $Header: /smsa/bsd:sys/conf/xcbconf.c:bsd.riscix1_2  1.16  $
 * $Source: /smsa/bsd:sys/conf/xcbconf.c: $
 *
 * Copyright (c) 1989 Acorn Computers Ltd., Cambridge, England
 *
 * $Desc$
 * $Log:	xcbconf.c,v $
 * Revision 1.16  93/03/23  17:01:57  kwelton
 * Added EtherIII drivers
 * 
 * Revision 1.15  91/10/31  18:21:29  kwelton
 * Added Hawk V12 driver.
 * 
 * Revision 1.14  90/09/07  11:04:24  mark
 * Added scavenge structure for ps (SCSI expansion card) driver.
 * 
 * Revision 1.13  90/08/16  09:41:33  mark
 * Added scavenging for both Ethernet card drivers and ST506 driver.
 * 
 * Revision 1.12  90/08/15  18:46:58  mark
 * Added suitable #includes and declarations to allow scavenge
 * stuff to compile.
 * 
 * Revision 1.11  90/08/15  15:54:28  mark
 * Added scavenge data pointers for the nym and hawk drivers.
 * 
 * Revision 1.10  90/08/07  11:10:55  kwelton
 * Added an entry for RAM/ROM card. There is no driver for this card, the
 * entry is present to remove the `unknown expansion card' message from
 * xcbman during bootstrap.
 * 
 * Revision 1.9  90/06/14  08:06:08  dave
 * added entries for hawkv10 driver
 * 
 * Revision 1.8  90/06/12  22:44:48  mark
 * Added "nym" driver (for Nymos 4-port serial card).
 * 
 * Revision 1.7  90/01/09  15:34:31  kwelton
 * Added an entry for Ethernet II cards
 * 
 * Revision 1.6  89/10/31  14:39:47  mark
 * Configuration now controlled by "devconf.h" rather than
 * by individual header files.
 * 
 * Revision 1.5  89/10/18  11:20:05  keith
 * Major shuffling of the header files, which now occur in ../include/... and no .h
 * files remain in source directories. Header files must now be included by
 * '#include <XYZ/name.h>' where XYZ is a sub-directory. For parametric header
 * files from the build directory use '#include "name.h"'
 * 
 * Revision 1.4  89/06/21  12:33:37  mark
 * Oops, I only made one out of the two fixes required for N{X}ST506.
 * 
 * Revision 1.3  89/06/21  11:26:52  mark
 * st driver handling modified reflecting changes in st.h.
 * 
 * Revision 1.2  89/05/02  09:05:22  kwelton
 * Added the routines for the new SCSI driver.
 * 
 * Revision 1.1  89/04/28  16:38:09  mark
 * Initial revision
 * 
 */

#if defined(KERN_RCS) && !defined(lint)
static char RcsId[] = "$Header: /smsa/bsd:sys/conf/xcbconf.c:bsd.riscix1_2  1.16  $";
#endif

/*
 * Expansion Card Bus configuration
 *
 * An entry must be included in xcb_type[] for every configured XCB
 * device driver.  A driver is identified by comparing the ID
 * information read from each installed expansion card with the
 * XCBInfo fields in xcb_type[].
 */
#include <sys/types.h>
#include <sys/conf.h>
#include <arm/xcb.h>
#include "devconf.h"

#if NET > 0
extern void et_init_high(), et_init_low(), et_shutdown();
extern struct scavenge et_scavenge;
#endif

#if NEN > 0
extern void en_init_high(), en_init_low(), en_shutdown();
extern struct scavenge en_scavenge;
#endif

#if NEA > 0
extern void ea_init_high(), ea_init_low(), ea_shutdown();
extern struct scavenge ea_scavenge;
#endif

#if NST506 > 0
extern void xst_init_high(), xst_shutdown();
extern struct scavenge st_scavenge;
#endif

#if NLBP > 0
extern void lbp_init_high(), lbp_shutdown();
#endif

#if PSC > 0
extern void ps_init_high(), ps_init_low(), ps_shutdown();
extern struct scavenge ps_scavenge;
#endif

#if NNYM > 0
extern void nym_init_hi(), nym_shutdown();
extern struct scavenge nym_scavenge;
#endif

#if NHAWK > 0
extern void hawk_init_high(), hawk_init_low(), hawk_shutdown();
extern struct scavenge hawk_scavenge;
#endif

#if NHAWKV12 > 0
extern void hawkv12_init_high(), hawkv12_init_low(), hawkv12_shutdown();
extern struct scavenge hawkv12_scavenge;
#endif

XCBType xcb_type[] =
{
#if NET > 0
  { { XCB_PRODUCT_ETHERNET, XCB_COMPANY_ACORNUK, XCB_COUNTRY_UK },
    0, et_init_high, et_init_low, et_shutdown, &et_scavenge },
#endif
#if NEN > 0
  { { XCB_PRODUCT_ETHERNET2, XCB_COMPANY_ACORNUK, XCB_COUNTRY_UK },
    0, en_init_high, en_init_low, en_shutdown, &en_scavenge },
#endif
#if NEA > 0
  { { XCB_PRODUCT_ATOMWIDEETHERNET, XCB_COMPANY_ATOMWIDELTD, XCB_COUNTRY_UK },
    0, ea_init_high, ea_init_low, ea_shutdown, &ea_scavenge },
#endif
#if NST506 > 0
  { { XCB_PRODUCT_WINCHESTER, XCB_COMPANY_ACORNUK, XCB_COUNTRY_UK },
    0, xst_init_high, 0, xst_shutdown, &st_scavenge },
#endif
#if NLBP > 0
  { { XCB_PRODUCT_LASERPRINTER, XCB_COMPANY_ACORNUK, XCB_COUNTRY_UK },
    0, lbp_init_high, 0, lbp_shutdown },
#endif
#if PSC > 0
 { { XCB_PRODUCT_SCSI, XCB_COMPANY_ACORNUK, XCB_COUNTRY_UK },
   0, ps_init_high, ps_init_low, ps_shutdown, &ps_scavenge },
#endif
#if NNYM > 0
  { { XCB_PRODUCT_QUADSERIAL, XCB_COMPANY_HUGHSYMONS, XCB_COUNTRY_UK },
    0, nym_init_hi, 0, nym_shutdown, &nym_scavenge },
#endif
#if NHAWK > 0
  { { XCB_PRODUCT_IMAGEANALYSER, XCB_COMPANY_WILDVISION, XCB_COUNTRY_UK },
    0, hawk_init_high, hawk_init_low, hawk_shutdown, &hawk_scavenge },
#endif
#if NHAWKV12 > 0
  { { XCB_PRODUCT_FRAMEGRABBER2, XCB_COMPANY_WILDVISION, XCB_COUNTRY_UK },
    0, hawkv12_init_high, hawkv12_init_low, hawkv12_shutdown, &hawkv12_scavenge },
#endif
  { { XCB_PRODUCT_RAMROM, XCB_COMPANY_ACORNUK, XCB_COUNTRY_UK },
    0, 0, 0, 0 },
};

int nxcb_type = sizeof(xcb_type)/sizeof(xcb_type[0]);

/* EOF xcbconf.c */
