/* iecs.h
 *
 * Header for Imaginary Expansion Card - Serial
 *
 * Copyright (C) 1989,1991 Acorn Computers Ltd.
 *
 */

/*
 * Header file defining the main programming interface of the IECS
 * card.  Some aspects of the individual ACIA chip (Asynchronous
 * Communications Interface Adaptor) interface are common with the
 * R140 internal serial port, so we use the system include file
 * <dev/acia.h> for those parts.
 */

/* Number of 65C51 ACIAs on each card */
#define LINES_PER_CARD  2

/*
 * Address of the ACIA register set (organised address-wise as for
 * the R140 internal 65C51) for each line.  This is relative to the
 * card slot start address in XCB I/O space - our card is operated
 * with medium-speed I/O cycles.
 */

#define CARD_ACIA(slot, line) \
    ((ACIARef)(XCB_ADDRESS(MEDIUM,slot) + (0x0800+0x200*(line))))

/*
 * Address of the IRQ register per card - this read/write byte-wide
 * register contains bits indicating the presence of IRQs for each
 * line (when read) and when written, the top bit controls the master
 * IRQ enable state - 0 means the card will not generate IRQs, 1 
 * allows it to.  The card IRQ line is the OR of all the individual
 * ACIA IRQ lines.  Writing to this register only affects the IRQ
 * enable bit, not the individual IRQ flags, so read-modify write 
 * coding is unnecessary.
 */

#define CARD_IRQ(slot) \
    ((volatile unsigned char *)(XCB_ADDRESS(MEDIUM,slot) + 0x4C0))

/* Define the interrupt enable bit */
#define IR_IENABLE      (1 << 7)

/* EOF iecs.h */
