/*
 * $Header: /smsa/bsd:sys/include/dev/hawkv12.h:bsd.riscix1_2  1.1  $
 * $Source: /smsa/bsd:sys/include/dev/hawkv12.h: $
 *
 * Copyright (c) 1991 Acorn Computers Ltd., Cambridge, England
 */
#ifndef __dev_hawkv12_h
#define __dev_hawkv12_h

/*
 * the hawkv12 is a double width card, which halves
 * the maximum number of cards the machine can hold
 */
#define HAWK_MAX	(XCB_SLOTS / 2)

/*
 * extract logical card number from device
 */
#define LOGICAL_SLOT(x)	((minor(x) & 0x80) >> 7)

/*
 * internal registers on TMS34061 Video System Controller
 */
typedef struct _vsc
{
    u_char HorizontalEndSync[2];
    u_char HorizontalEndBlank[2];
    u_char HorizontalStartBlank[2];
    u_char HorizontalTotal[2];
    u_char VerticalEndSync[2];
    u_char VerticalEndBlank[2];
    u_char VerticalStartBlank[2];
    u_char VerticalTotal[2];
    u_char DisplayAddressUpdate[2];
    u_char DisplayStart[2];
    u_char VerticalInterrupt[2];
    u_char Control1[2];
    u_char Control2[2];
    u_char StatusReg[2];
    u_char X_YOffset[2];
    u_char X_YAddress[2];
    u_char DisplayAddress[2];
    u_char VerticalCount[2];
} VSC, *VSCRef;

#define VSC_NREGS	((sizeof(VSC) / sizeof(u_char)) / 2)
#define	Low		0
#define	High		1

#define CLUT_PIXEL_ENTRIES	(256)
#define CLUT_OVERLAY_ENTRIES	(16)		/* NOTE: entry 0 is invalid */
#define CLUT_BYTES_PER_ENTRY	(3)

typedef struct _clut
{
    u_short PixelNEntries;
    u_char PixelStartAddr;
    u_char PixelReadMask;
    u_short OverlayNEntries;
    u_char OverlayStartAddr;
    u_char PixelData[CLUT_PIXEL_ENTRIES * CLUT_BYTES_PER_ENTRY];
    u_char OverlayData[(CLUT_OVERLAY_ENTRIES - 1) * CLUT_BYTES_PER_ENTRY];
} CLUT, *CLUTRef;

typedef struct _frames
{
    u_char read;
    u_char write;
    u_char display;
} Frames, *FramesRef;

/*
 * ioctls specific to hawkv12 driver
 */
#define HAWKSVDIRN	_IOW('h', 0, int)	/* set video direction */
#define DISPLAY		(1)
#define GRAB		(2)
#define SNAPSHOT	(3)

#define HAWKGCTRL	_IOR('h', 1, int)	/* get hawk ctrl register */
#define HAWKSCTRL	_IOW('h', 2, int)	/* set hawk ctrl register */

#define HAWKGWMASK	_IOR('h', 3, int)	/* get write mask register */
#define HAWKSWMASK	_IOW('h', 4, int)	/* set write mask register */

#define HAWKGVSCMASK	_IOR('h', 5, VSC)	/* get VSC user mask */
#define HAWKGVSCREGS	_IOR('h', 6, VSC)	/* get VSC registers */
#define HAWKSVSCREGS	_IOW('h', 7, VSC)	/* set VSC registers */

#define HAWKGFRAMES	_IOR('h', 8, Frames)	/* get read/write/display
						 * frames */
#define HAWKSFRAMES	_IOW('h', 9, Frames)	/* set read/write/display
						 * frames */

/*
 * NOTE: the following two ioctl()s use a structure that is
 * too big to be passed directly across the ioctl interface,
 * so take a pointer to the structure and do the copy within
 * the driver.
 */
#define HAWKGCLUT	_IOW('h', 10, CLUTRef)	/* get CLUT data. */
#define HAWKSCLUT	_IOW('h', 11, CLUTRef)	/* set CLUT data. */

#define HAWKRESET	_IOW('h', 12, int)	/* reset the card */
#define HAWKCLEAR	_IOW('h', 13, int)	/* clear frames */

#endif /* !defined(__dev_hawkv12_h) */

/* EOF hawkv12.h */
