/*
 * $Header: /smsa/bsd:sys/include/sys/ttydev.h:bsd.riscix1_2  1.10  $
 * $Source: /smsa/bsd:sys/include/sys/ttydev.h: $
 *
 * Copyright (c) 1988 Acorn Computers Ltd., Cambridge, England
 *
 */
/*	@(#)ttydev.h	2.1 88/05/18 4.0NFSSRC SMI;	*/
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)ttydev.h	7.1 (Berkeley) 6/4/86
 */

/*
 * Terminal definitions related to underlying hardware.
 */
#ifndef	__sys_ttydev_h
#define	__sys_ttydev_h

/*
 * Speeds: #define lines have been made identical to those in
 * <termios.h>, to avoid compiler complaints when #including
 * both headers.
 */
#define B0		0
#define B50		1
#define B75		2
#define B110		3
#define B134		4
#define B150		5
#define B200		6
#define B300		7
#define B600		8
#define B1200		9
#define	B1800		10
#define B2400		11
#define B4800		12
#define B9600		13
#define B19200		14
#define EXTA		14
#define B38400		15
#define EXTB		15

#ifdef _KERNEL
/*
 * Modem control commands.
 */
#define	DMSET		0
#define	DMBIS		1
#define	DMBIC		2
#define	DMGET		3

/*
 * serial line buffering control
 */
#define BASELINE_SPEED	B9600
#define ARM2_BASELINE	384
#define ARM3_BASELINE	128

/*
 * per-device FIFO, transmit buffer,
 * and handler routines
 */
struct serial_fifo
{
    u_char      *sf_fifo;
    u_char      *sf_fill;
    u_char      *sf_drain;
    unsigned     sf_size;
    volatile int sf_count;
    volatile int sf_dropped;
    u_char      *sf_txnext;
    u_char      *sf_txlimit;
    void       (*sf_rxfn)();
    void       (*sf_txfn)();
};

/*
 * Each entry in the FIFO consists of a control byte, and
 * (potentially) a data byte.  The control byte contains the
 * interrupt type, modem status, and received character
 * status; this is all packed into a single char in an
 * attempt to minimise the required buffer size.
 * Unfortunately, due to lack of space, some bits within
 * the control byte are overloaded with transmit/receive
 * status.
 */

/*
 * control byte types
 */
#define FIFO_TYPE_MASK		0x03
#define FIFO_RX			0x00			/* Rx'd char */
#define FIFO_TX			0x01			/* Tx'd char */
#define FIFO_ENTRY_TYPE(x)	((x) & FIFO_TYPE_MASK)

/*
 * control byte received character status
 */
#define FIFO_FRMERR	(1 << 4)
#define FIFO_PARERR	(1 << 5)
#define FIFO_OVERRUN	(1 << 6)
#define FIFO_BREAK	(1 << 7)

/*
 * control byte transmitted character type
 */
#define FIFO_DOFLUSH	(1 << 4)

/*
 * bitmaps for the generic SOFTSERIAL software
 * irq handler to identify which drivers have
 * a software irq pending
 */
#define SOFT_ES		(1 << 0)
#define SOFT_ZILOG	(1 << 1)
#define SOFT_NYMOS	(1 << 2)

#endif /* _KERNEL */

#endif/*__sys_ttydev_h*/

/* EOF ttydev.h */
