/*
 * $Header: /smsa/bsd:sys/sys/tty_conf.c:bsd.riscix1_2  1.13  $
 * $Source: /smsa/bsd:sys/sys/tty_conf.c: $
 *
 * Copyright (c) 1988 Acorn Computers Ltd., Cambridge, England
 *
 * $Desc$
 * $Log:	tty_conf.c,v $
 * Revision 1.13  93/03/23  17:17:11  kwelton
 * Added PPP support.  This includes initialisation of new l_select field in
 * linesw for all existing line disciplines.
 * 
 * Revision 1.12  90/11/08  18:20:38  mark
 * Mouse/keyboard input line discipline now has its own read
 * function to avoid problems over POSIX tty interface.
 * 
 * Revision 1.11  90/07/09  09:15:48  mark
 * Added a POSIX line discipline (using same entry points as old/new 
 * tty disciplines).
 * 
 * Revision 1.10  89/10/31  15:14:24  mark
 * Define NBK as 0 if it isn't already defined.  We aren't really
 * interested in supporting BKNet line discipline...
 * 
 * Revision 1.9  89/10/30  16:40:33  mark
 * Line configuration is now specified by "lineconf.h" in 
 * the kernel build directory: this is (more or less) the
 * only file which includes it.
 * 
 * Revision 1.8  89/10/18  11:08:07  keith
 * Major shuffling of the header files, which now occur in ../include/... and no .h
 * files remain in source directories. Header files must now be included by
 * '#include <XYZ/name.h>' where XYZ is a sub-directory. For parametric header
 * files from the build directory use '#include "name.h"'
 * 
 * Revision 1.7  89/09/28  19:16:22  keith
 * Dummy assignments to remove warnings
 * 
 * Revision 1.6  89/09/27  09:32:41  keith
 * RISC iX + NFS 4.0
 * 
 * Revision 1.4  89/04/28  14:41:54  mark
 * Extensions to MKIDISC functionality.
 * 
 * Revision 1.3  88/06/18  09:58:57  beta
 * Acorn Unix initial beta version
 * 
 */

#if defined(KERN_RCS) && !defined(lint)
static char RcsId[] = "$Header: /smsa/bsd:sys/sys/tty_conf.c:bsd.riscix1_2  1.13  $";
#endif
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)tty_conf.c	7.1 (Berkeley) 6/5/86
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/buf.h>
#include <sys/ioctl.h>
#include <sys/tty.h>
#include <sys/conf.h>

#include "lineconf.h"

int	nodev();
int	nulldev();

int	ttyopen(),ttylclose(),ttread(),ttwrite(),nullioctl(),ttstart();
int	ttymodem(), nullmodem(), ttyinput(), ttyselect();

#ifndef NBK
#define NBK 0
#endif

#if NBK > 0
int	bkopen(),bkclose(),bkread(),bkinput(),bkioctl();
#endif

#if NTB > 0
int	tbopen(),tbclose(),tbread(),tbinput(),tbioctl();
#endif

#if NSL > 0
int	slopen(),slclose(),slinput(),sltioctl(),slstart();
#endif

#if NMKI > 0
int	mkiopen(),mkiclose(),mkiinput(),mkiread();
#endif

#if NPPP > 0
int	pppopen(),pppclose(),pppread(),pppwrite(),pppinput();
int	ppptioctl(),pppstart(),pppselect();
#endif

struct	linesw linesw[] =
{
{
	ttyopen, ttylclose, ttread, ttwrite, nullioctl,	/* 0- OTTYDISC */
	ttyinput, nodev, nulldev, ttstart, ttymodem, ttyselect,
    },
{
#if NBK > 0
	bkopen, bkclose, bkread, ttwrite, bkioctl,	/* 1- NETLDISC */
	bkinput, nodev, nulldev, ttstart, nullmodem, ttyselect,
#else
	nodev, nodev, nodev, nodev, nodev,
	nodev, nodev, nodev, nodev, nodev, nodev,
#endif
    },
{
	ttyopen, ttylclose, ttread, ttwrite, nullioctl,	/* 2- NTTYDISC */
	ttyinput, nodev, nulldev, ttstart, ttymodem, ttyselect,
    },
{
#if NTB > 0
	tbopen, tbclose, tbread, nodev, tbioctl,	/* 3- TABLDISC */
	tbinput, nodev, nulldev, ttstart, nullmodem, ttyselect,
#else
	nodev, nodev, nodev, nodev, nodev,
	nodev, nodev, nodev, nodev, nodev, nodev,
#endif
    },
{
#if NSL > 0
	slopen, slclose, nodev, nodev, sltioctl,	/* 4- SLIPDISC */
	slinput, nodev, nulldev, slstart, nulldev, ttyselect,
#else
	nodev, nodev, nodev, nodev, nodev,
	nodev, nodev, nodev, nodev, nodev, nodev,
#endif
    },
{
#if NMKI > 0
	mkiopen, mkiclose, mkiread, ttwrite, nullioctl,	/* 5- MKIDISC */
	mkiinput, nodev, nulldev, ttstart, ttymodem, ttyselect,
#else
	nodev, nodev, nodev, nodev, nodev,
	nodev, nodev, nodev, nodev, nodev, nodev,
#endif
    },
{
	ttyopen, ttylclose, ttread, ttwrite, nullioctl,	/* 6- POSIXDISC */
	ttyinput, nodev, nulldev, ttstart, ttymodem, ttyselect,
    },
{
#if NPPP > 0
	pppopen, pppclose, pppread, pppwrite, ppptioctl,/* 7- PPPDISC */
	pppinput, nodev, nulldev, pppstart, ttymodem, pppselect,
#else
	nodev, nodev, nodev, nodev, nodev,
	nodev, nodev, nodev, nodev, nodev, nodev,
#endif
},
};

int	nldisp = sizeof (linesw) / sizeof (linesw[0]);

/*
 * Do nothing specific version of line
 * discipline specific ioctl command.
 */
/*ARGSUSED*/
nullioctl(tp, cmd, data, flags)
	struct tty *tp;
	char *data;
	int flags;
{
	tp = tp; cmd = cmd; data = data; flags = flags;

	return (-1);
}
